/*
 *  uc1701_v1.h
 *  Created on: 19.5.2025
 *  Author: misa
 *
 *  Jednoduché rozhraní pro displeje s driverem UC1701 a ST7565R
 *
 *  Připojení:
 *  SCK (SPI clock)
 *  MOSI (SPI data)
 *  CD(RS) Command (Low)/Data (High)
 *  CS Chip Select
 *  RST trvale do High (využívá se Reset po SPI)
 *  Napájení 2.5-3.3V
 *  Max přenosová rychlost 33MHz (SPI MODE3 - clock ve výchozím stavu je v log.1, driver čte na vzestupnou hranu clocku)
 *  Min Setup/Hold time jsou: MOSI (12/0ns), CS (5/5ns), CD (0/0ns)
 *
 *  Celý obraz se udržuje v paměti jako pole 128x64 bit, tedy 128x8=1kB (pro displeje s jiným rozlišením může být buffer jinak velký)
 *  styk s displejem provádějí funkce:
 *  uc1701_init(); // inicializuje HW, resetuje a inicializuje displej
 *  uc1701_display(); // přepíše kompletní obraz displeje (argumentem je obrazový buffer)
 *  veškerá grafika by se měla provádět nad bufferem
 *
 *
 *	Rychlost zobrazování bit-bang varianty:
 *	STM32C0, 48MHz, -Og, 	 4.4ms
 *	STM32C0, 48MHz, -fast, 	 2.7ms
 *	STM32C0, 12MHz, -Og, 	15.4ms
 *
 *
 * Pokyny pro portování:
 * a) v souboru uc1701_LLv1.c ve funkci init_bus() upravit kód tak aby inicializoval HW prostředky vašeho MCU
 * b) v souboru uc1701_LLv1.h upravit makra UC1701_CS_x a UC1701_RS_x tak aby na vašem MCU nastavovali CS a CD(RS) do log.1 a log.0
 * c) v souboru uc1701_LLv1.c funkci uc1701_command() přepište tak aby prováděla přenos 1B s CD(RS)=1. Více v komentáři v záhlaví funkce.
 * d) v souboru uc1701_LLv1.c ve funkci uc1701_display() přepište sekci která posílá obsah jednoho řádku (8px) displeje. Více v komentáři v záhlaví funkce.
 * e) v případně displeje s jiným rozlišením můžete změňte FRAME_WIDTH,FRAME_PX_HEIGHT,FRAME_HEIGHT
 *
 *
 * Displej GMG12864 má rozměry okénka pouze 48x24mm (okraje bez obrazu jsou 4 a 6mm).
 * Displej JLX12864 má rozměry okénka --------
 *
 * editlog:
 * released 19.5.2025
 */

#ifndef UC1701_V1_H_
#define UC1701_V1_H_

// inkluze potřebných knihoven pro vybrané MCU. Zde pro STM32 s LL API
#include "main.h"

#define UC1701_BITBANG // volí "bit-bang" variantu (časem snad přibude HW SPI varianta)

// 1. zvol typ řadiče (ST7565R vs UC1701) odkomentováním vybraného makra
// 2. namapuj piny CS,RS,SCK,MOSI pomocí maker
// 3. uprav makro PORT_CLOCK_ENABLE aby zapnul clock všem použitým portům, nebo si ho zapni sám před voláním fce uc1701_init()

// volba driveru (ovlivňuje inicializační nastavení kontrastů a electronic volume)
#define ST7565R
// #define UC1701

#define UC1701_CS_GPIO			GPIOA
#define UC1701_CS_PIN 			LL_GPIO_PIN_1
#define UC1701_RS_GPIO			GPIOA
#define UC1701_RS_PIN 			LL_GPIO_PIN_3
#define UC1701_SCK_GPIO		GPIOA
#define UC1701_SCK_PIN 		LL_GPIO_PIN_5
#define UC1701_MOSI_GPIO	GPIOA
#define UC1701_MOSI_PIN 	LL_GPIO_PIN_2

#define PORT_CLOCK_ENABLE LL_IOP_GRP1_EnableClock(LL_IOP_GRP1_PERIPH_GPIOA)
//#define PORT_CLOCK_ENABLE LL_IOP_GRP1_EnableClock(LL_IOP_GRP1_PERIPH_GPIOA | LL_IOP_GRP1_PERIPH_GPIOB)  // příklad úpravy

// Private macros

#define FRAME_HEIGHT 8 // pages
#define FRAME_PX_HEIGHT 64 // height in pixels
#define FRAME_WIDTH 128 // columns

#define LCD_RESET												0xE2 // Po resetu je teoreticky nutno 6ms počkat
#define LCD_DISPLAY_ENABLE  	0xAF	// aktivuje displej
#define LCD_DISPLAY_DISABLE 	0xAE	// deaktivuje displej (přejde do režimu spánku)
#define LCD_START_LINE							0x40 // scrollování - na nic asi ... počítám že vertikální shift displeje (ve smyslu pages, ale po bitech)
#define LCD_PAGE_ADDRESS    	0xB0
#define LCD_COL_ADDRESS_MSB 	0x10
#define LCD_COL_ADDRESS_LSB 	0x00
#define LCD_SEGDIR											0xA0 //
#define LCD_DISPLAY_INVERT  	0xA6 // funkci aktivujte přidáním (přiořením) jedničky na nejnižší bit
#define LCD_ALL_PIXEL	 						0xA4 // funkci aktivujte přidáním (přiořením) jedničky na nejnižší bit
#define LCD_BIAS	      						0xA2 // nechat takhle !
#define LCD_COMDIR	  								0xC0 // zrcadlení displeje podle krátké osy (aktivuje se hodnotou 1<<3, deaktivuje 0)
#define LCD_POWER_CONTROL   	0x28 // zapnout šťávu
#define LCD_RATIO	   								0x20 // kontrast - hodně citlivé :/
#define LCD_VOLUME       				0x81 // něco kouzelného - hodně citlivé
#define LCD_NO_OP	     						0xE3 // asi na nic (daisy chain ?)
#define LCD_ADV_PROG_CTRL0  	0xFA // umožňuje zapnout vrapování a vybírat temp coef ... dvoubajtový příkaz
#define LCD_ADV_PROG_CTRL2  	0x10 // sem se nesmí zapisovat :D !
#define LCD_BOOSTER_RATIO				0xF8 // dvoubajtový příkaz (druhá hodnota 00=2x3x4x, 01=5x, 02=6x);

#define UC1701_CS_H 		LL_GPIO_SetOutputPin(UC1701_CS_GPIO,UC1701_CS_PIN)   // CS Chip Select pin
#define UC1701_CS_L 		LL_GPIO_ResetOutputPin(UC1701_CS_GPIO,UC1701_CS_PIN) // CS Chip Select pin
#define UC1701_RS_H 		LL_GPIO_SetOutputPin(UC1701_RS_GPIO,UC1701_RS_PIN)   // CD Command/Data pin
#define UC1701_RS_L 		LL_GPIO_ResetOutputPin(UC1701_RS_GPIO,UC1701_RS_PIN) // CD Command/Data pin
#define UC1701_SCK_H 	LL_GPIO_SetOutputPin(UC1701_SCK_GPIO,UC1701_SCK_PIN)
#define UC1701_SCK_L 	LL_GPIO_ResetOutputPin(UC1701_SCK_GPIO,UC1701_SCK_PIN)
#define UC1701_MOSI_H LL_GPIO_SetOutputPin(UC1701_MOSI_GPIO,UC1701_MOSI_PIN)
#define UC1701_MOSI_L LL_GPIO_ResetOutputPin(UC1701_MOSI_GPIO,UC1701_MOSI_PIN)

void uc1701_init(void);
void uc1701_data(uint8_t data);
void uc1701_command(uint8_t command);
void uc1701_display(uint8_t* array);
void uc1701_create_test_pattern(uint8_t* image_buffer);

#define uc1701_reset() 													uc1701_command(LCD_RESET)
#define uc1701_contrast(k) 									uc1701_command(LCD_RATIO | (k & 0b111))
#define uc1701_allpixel_on() 							uc1701_command(LCD_ALL_PIXEL|1)
#define uc1701_allpixel_off() 						uc1701_command(LCD_ALL_PIXEL|0)
#define uc1701_invert_on() 									uc1701_command(LCD_DISPLAY_INVERT|1)
#define uc1701_invert_off() 								uc1701_command(LCD_DISPLAY_INVERT|0)
#define uc1701_vertical_flip_on() 		uc1701_command(LCD_COMDIR | 1<<3)
#define uc1701_vertical_flip_off() 	uc1701_command(LCD_COMDIR | 0<<3)

#endif /* UC1701_V1_H_ */
